package com.williamluke4.safe_sms

import android.Manifest
import android.content.pm.PackageManager
import android.database.Cursor
import android.net.Uri

import com.williamluke4.safe_sms.permisions.Permissions

import org.json.JSONException
import org.json.JSONObject

import java.util.ArrayList

import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry

import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.RequestPermissionsResultListener

/**
 * Created by babariviere on 09/03/18.
 */

internal enum class SmsQueryRequest {
    Inbox,
    Sent,
    Draft;


    fun toUri(): Uri {
        return if (this == Inbox) {
            Uri.parse("content://sms/inbox")
        } else if (this == Sent) {
            Uri.parse("content://sms/sent")
        } else {
            Uri.parse("content://sms/draft")
        }
    }
}

internal class SmsQueryHandler(private val registrar: PluginRegistry.Registrar, private val result: MethodChannel.Result, private val request: SmsQueryRequest,
                               start: Int, count: Int, threadId: Int, address: String) : RequestPermissionsResultListener {
    private val permissionsList = arrayOf<String>(Manifest.permission.READ_SMS)
    private var start = 0
    private var count = -1
    private val threadId = -1
    private val address: String? = null

    init {
        this.start = start
        this.count = count
        this.threadId = threadId
        this.address = address
    }

    fun handle(permissions: Permissions) {
        if (permissions.checkAndRequestPermission(permissionsList, Permissions.SEND_SMS_ID_REQ)) {
            querySms()
        }
    }

    private fun readSms(cursor: Cursor): JSONObject {
        val res = JSONObject()
        for (idx in 0 until cursor.getColumnCount()) {
            try {
                if (cursor.getColumnName(idx).equals("address") || cursor.getColumnName(idx).equals("body")) {
                    res.put(cursor.getColumnName(idx), cursor.getString(idx))
                } else if (cursor.getColumnName(idx).equals("date") || cursor.getColumnName(idx).equals("date_sent")) {
                    res.put(cursor.getColumnName(idx), cursor.getLong(idx))
                } else {
                    res.put(cursor.getColumnName(idx), cursor.getInt(idx))
                }
            } catch (e: JSONException) {
                e.printStackTrace()
            }

        }
        return res
    }

    private fun querySms() {
        val list = ArrayList()
        val cursor = registrar.context().getContentResolver().query(this.request.toUri(), null, null, null, null)
        if (cursor == null) {
            result.error("#01", "permission denied", null)
            return
        }
        if (!cursor!!.moveToFirst()) {
            cursor!!.close()
            result.success(list)
            return
        }
        do {
            val obj = readSms(cursor!!)
            try {
                if (threadId >= 0 && obj.getInt("thread_id") !== threadId) {
                    continue
                }
                if (address != null && !obj.getString("address").equals(address)) {
                    continue
                }
            } catch (e: JSONException) {
                e.printStackTrace()
            }

            if (start > 0) {
                start--
                continue
            }
            list.add(obj)
            if (count > 0) {
                count--
            }
        } while (cursor!!.moveToNext() && count != 0)
        cursor!!.close()
        result.success(list)
    }

    @Override
    fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray): Boolean {
        if (requestCode != Permissions.READ_SMS_ID_REQ) {
            return false
        }
        var isOk = true
        for (res in grantResults) {
            if (res != PackageManager.PERMISSION_GRANTED) {
                isOk = false
                break
            }
        }
        if (isOk) {
            querySms()
            return true
        }
        result.error("#01", "permission denied", null)
        return false
    }
}

internal class SmsQuery(private val registrar: PluginRegistry.Registrar) : MethodCallHandler {
    private val permissions: Permissions


    init {
        this.permissions = Permissions(registrar.activity())
    }

    @Override
    fun onMethodCall(call: MethodCall, result: Result) {
        var start = 0
        var count = -1
        var threadId = -1
        var address: String? = null
        val request: SmsQueryRequest
        when (call.method) {
            "getInbox" -> request = SmsQueryRequest.Inbox
            "getSent" -> request = SmsQueryRequest.Sent
            "getDraft" -> request = SmsQueryRequest.Draft
            else -> {
                result.notImplemented()
                return
            }
        }
        if (call.hasArgument("start")) {
            start = call.argument("start")
        }
        if (call.hasArgument("count")) {
            count = call.argument("count")
        }
        if (call.hasArgument("thread_id")) {
            threadId = call.argument("thread_id")
        }
        if (call.hasArgument("address")) {
            address = call.argument("address")
        }
        val handler = SmsQueryHandler(registrar, result, request, start, count, threadId, address)
        this.registrar.addRequestPermissionsResultListener(handler)
        handler.handle(permissions)
    }

}
