package com.williamluke4.safe_sms.permisions

import android.app.Activity
import android.content.pm.PackageManager
import android.os.Build

import io.flutter.plugin.common.PluginRegistry

/**
 * Created by babariviere on 08/03/18.
 */

class Permissions(private val activity: Activity) {

    private fun hasPermission(permission: String): Boolean {
        return Build.VERSION.SDK_INT < Build.VERSION_CODES.M || activity.checkSelfPermission(permission) === PackageManager.PERMISSION_GRANTED
    }

    private fun hasPermissions(permissions: Array<String>): Boolean {
        for (perm in permissions) {
            if (!hasPermission(perm)) {
                return false
            }
        }
        return true
    }

    fun checkAndRequestPermission(permissions: Array<String>, id: Int): Boolean {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            return true
        }
        if (!hasPermissions(permissions)) {
            PermissionsRequestHandler.requestPermissions(
                    PermissionsRequest(id, permissions, activity)
            )
            return false
        }
        return true
    }

    companion object {
        val READ_SMS_ID_REQ = 3
        private val requestsListener = PermissionsRequestHandler()

        val requestsResultsListener: PluginRegistry.RequestPermissionsResultListener
            get() = requestsListener
    }
}
